@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2020 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2021 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -26,9 +26,9 @@ import ipaddress
 import re
 import socket
 
-from collections import OrderedDict
 from pathlib import Path
 from random import SystemRandom
+from typing import Union
 
 from .constants import DEFAULT_LANG
 from .constants import DISC_COMPRESSION_ERROR, DISC_CONNECTION_LOST
@@ -39,6 +39,9 @@ from .constants import DISC_PROTOCOL_ERROR, DISC_PROTOCOL_VERSION_NOT_SUPPORTED
 from .constants import DISC_SERVICE_NOT_AVAILABLE
 
 
+BytesOrStr = Union[bytes, str]
+
+
 # Define a version of randrange which is based on SystemRandom(), so that
 # we get back numbers suitable for cryptographic use.
 _random = SystemRandom()
@@ -150,6 +153,20 @@ def open_file(filename, *args, **kwargs):
     return open(Path(filename).expanduser(), *args, **kwargs)
 
 
+def read_file(filename, mode='rb'):
+    """Read from a file with home directory expansion"""
+
+    with open_file(filename, mode) as f:
+        return f.read()
+
+
+def write_file(filename, data, mode='wb'):
+    """Write or append to a file with home directory expansion"""
+
+    with open_file(filename, mode) as f:
+        return f.write(data)
+
+
 def _parse_units(value, suffixes, label):
     """Parse a series of integers followed by unit suffixes"""
 
@@ -256,11 +273,31 @@ class Options:
     def prepare(self):
         """Pre-process configuration options"""
 
+    def update(self, kwargs):
+        """Update options based on keyword parameters passed in"""
 
-class Record:
+        self.kwargs.update(kwargs)
+        self.prepare(**self.kwargs)
+
+
+class _RecordMeta(type):
+    """Metaclass for general-purpose record type"""
+
+    def __new__(mcs, name, bases, ns):
+        if name != 'Record':
+            fields = ns.get('__annotations__', {}).keys()
+            defaults = {k: ns.get(k) for k in fields}
+
+            ns = {k: v for k, v in ns.items() if k not in fields}
+            ns['__slots__'] = defaults
+
+        return super().__new__(mcs, name, bases, ns)
+
+
+class Record(metaclass=_RecordMeta):
     """General-purpose record type with fixed set of fields"""
 
-    __slots__ = OrderedDict()
+    __slots__ = {}
 
     def __init__(self, *args, **kwargs):
         for k, v in self.__slots__.items():
