@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -162,7 +162,6 @@ class _EchoServerSession(asyncssh.SSHServerSession):
 
     def __init__(self):
         self._chan = None
-        self._pty_ok = True
 
     def connection_made(self, chan):
         """Handle session open"""
@@ -173,16 +172,9 @@ class _EchoServerSession(asyncssh.SSHServerSession):
 
         if username == 'close':
             self._chan.close()
-        elif username == 'no_pty':
-            self._pty_ok = False
         elif username == 'task_error':
             raise RuntimeError('Exception handler test')
 
-    def pty_requested(self, term_type, term_size, term_modes):
-        """Handle pseudo-terminal request"""
-
-        return self._pty_ok
-
     def shell_requested(self):
         """Handle shell request"""
 
@@ -203,6 +195,47 @@ class _EchoServerSession(asyncssh.SSHServerSession):
         self._chan.close()
 
 
+class _PTYServerSession(asyncssh.SSHServerSession):
+    """Server for testing PTY requests"""
+
+    def __init__(self):
+        self._chan = None
+        self._pty_ok = True
+
+    def connection_made(self, chan):
+        """Handle session open"""
+
+        self._chan = chan
+
+        username = self._chan.get_extra_info('username')
+
+        if username == 'no_pty':
+            self._pty_ok = False
+
+    def pty_requested(self, term_type, term_size, term_modes):
+        """Handle pseudo-terminal request"""
+
+        self._chan.set_extra_info(
+            pty_args=(term_type, term_size,
+                      term_modes.get(asyncssh.PTY_OP_OSPEED)))
+
+        return self._pty_ok
+
+    def shell_requested(self):
+        """Handle shell request"""
+
+        return True
+
+    def session_started(self):
+        """Handle session start"""
+
+        chan = self._chan
+
+
+        chan.write(f'Req: {chan.get_extra_info("pty_args")}\n')
+        chan.close()
+
+
 class _ChannelServer(Server):
     """Server for testing the AsyncSSH channel API"""
 
@@ -250,8 +283,8 @@ class _ChannelServer(Server):
             except asyncssh.ChannelOpenError:
                 stdout.channel.exit(1)
         elif action == 'rejected_session':
-            chan = _ServerChannel(self._conn, asyncio.get_event_loop(),
-                                  False, False, 0, None, 'strict', 1, 32768)
+            chan = _ServerChannel(self._conn, asyncio.get_event_loop(), False,
+                                  False, 0, 1024, None, 'strict', 1, 32768)
 
             try:
                 await chan.open_session()
@@ -411,7 +444,8 @@ class _ChannelServer(Server):
 
         return username not in {'guest', 'conn_close_startup',
                                 'conn_close_open', 'close', 'echo',
-                                'no_channels', 'no_pty', 'task_error'}
+                                'no_channels', 'no_pty', 'request_pty',
+                                'task_error'}
 
     def session_requested(self):
         """Handle a request to create a new session"""
@@ -426,8 +460,10 @@ class _ChannelServer(Server):
                 return False
             elif username == 'conn_close_open':
                 return (channel, self._conn_close())
-            elif username in {'close', 'echo', 'no_pty', 'task_error'}:
+            elif username in {'close', 'echo', 'task_error'}:
                 return (channel, _EchoServerSession())
+            elif username in {'request_pty', 'no_pty'}:
+                return (channel, _PTYServerSession())
             elif username != 'no_channels':
                 return (channel, self._begin_session)
             else:
@@ -456,12 +492,11 @@ class _TestChannel(ServerTestCase):
 
             self.assertEqual(session.exit_status, expected_result)
 
-    async def _check_session(self, conn, command=None, *, subsystem=None,
+    async def _check_session(self, conn, command=(), *,
                              large_block=False, **kwargs):
         """Open a session and test if an input line is echoed back"""
 
-        chan, session = await _create_session(conn, command,
-                                              subsystem=subsystem, *kwargs)
+        chan, session = await _create_session(conn, command, **kwargs)
 
         if large_block:
             data = 4 * [1025*1024*'\0']
@@ -519,7 +554,15 @@ class _TestChannel(ServerTestCase):
         """Test execution of a remote command"""
 
         async with self.connect() as conn:
-            await self._check_session(conn, 'echo')
+            await self._check_session(conn, 'echo', window=1024*1024,
+                                      max_pktsize=16384)
+
+    @asynctest
+    async def test_exec_from_connect(self):
+        """Test execution of a remote command set on connection"""
+
+        async with self.connect(command='echo') as conn:
+            await self._check_session(conn)
 
     @asynctest
     async def test_forced_exec(self):
@@ -530,8 +573,8 @@ class _TestChannel(ServerTestCase):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 options={'force-command': String('echo')})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             await self._check_session(conn)
 
     @asynctest
@@ -861,7 +904,7 @@ class _TestChannel(ServerTestCase):
                                 extensions={'no-agent-forwarding': ''})
 
         async with self.connect(username='ckey', client_keys=[(ckey, cert)],
-                                agent_forwarding=True) as conn:
+                                agent_path=None, agent_forwarding=True) as conn:
             chan, session = await _create_session(conn, 'agent')
 
             await chan.wait_closed()
@@ -913,21 +956,20 @@ class _TestChannel(ServerTestCase):
             self.assertEqual(session.exit_status, 1)
 
     @asynctest
-    async def test_terminal_info(self):
-        """Test sending terminal information"""
+    async def test_request_pty(self):
+        """Test reuquesting a PTY with terminal information"""
 
         modes = {asyncssh.PTY_OP_OSPEED: 9600}
 
-        async with self.connect() as conn:
-            chan, session = await _create_session(conn, 'term',
-                                                  term_type='ansi',
+        async with self.connect(username='request_pty') as conn:
+            chan, session = await _create_session(conn, term_type='ansi',
                                                   term_size=(80, 24),
                                                   term_modes=modes)
 
             await chan.wait_closed()
 
             result = ''.join(session.recv_buf[None])
-            self.assertEqual(result, "('ansi', (80, 24, 0, 0), 9600)\r\n")
+            self.assertEqual(result, "Req: ('ansi', (80, 24, 0, 0), 9600)\r\n")
 
     @asynctest
     async def test_terminal_full_size(self):
@@ -988,8 +1030,8 @@ class _TestChannel(ServerTestCase):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 extensions={'no-pty': ''})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await _create_session(conn, 'term', term_type='ansi')
 
@@ -999,7 +1041,7 @@ class _TestChannel(ServerTestCase):
 
         async with self.connect(username='no_pty') as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
-                await _create_session(conn, 'term', term_type='ansi')
+                await _create_session(conn, term_type='ansi')
 
     @asynctest
     async def test_invalid_term_type(self):
@@ -1052,6 +1094,18 @@ class _TestChannel(ServerTestCase):
             result = ''.join(session.recv_buf[None])
             self.assertEqual(result, 'test\n')
 
+    @asynctest
+    async def test_env_from_connect(self):
+        """Test setting environment on connection"""
+
+        async with self.connect(env={'TEST': 'test'}) as conn:
+            chan, session = await _create_session(conn, 'env')
+
+            await chan.wait_closed()
+
+            result = ''.join(session.recv_buf[None])
+            self.assertEqual(result, 'test\n')
+
     @asynctest
     async def test_env_list(self):
         """Test setting environment using a list of name=value strings"""
@@ -1090,6 +1144,23 @@ class _TestChannel(ServerTestCase):
             result = ''.join(session.recv_buf[None])
             self.assertEqual(result, 'test\n')
 
+    @asynctest
+    async def test_send_env_from_connect(self):
+        """Test sending local environment on connection"""
+
+        try:
+            os.environ['TEST'] = 'test'
+
+            async with self.connect(send_env=['TEST']) as conn:
+                chan, session = await _create_session(conn, 'env')
+
+                await chan.wait_closed()
+
+                result = ''.join(session.recv_buf[None])
+                self.assertEqual(result, 'test\n')
+        finally:
+            del os.environ['TEST']
+
     @asynctest
     async def test_mixed_env(self):
         """Test sending a mix of local environment and new values"""
@@ -1526,6 +1597,14 @@ class _TestChannelNoPTY(ServerTestCase):
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await conn.run('echo', request_pty='force')
 
+    @asynctest
+    async def test_exec_pty_from_connect(self):
+        """Test execution of a command that requests a PTY on the connection"""
+
+        async with self.connect(request_pty='force') as conn:
+            with self.assertRaises(asyncssh.ChannelOpenError):
+                await conn.run('echo')
+
     @asynctest
     async def test_exec_no_pty(self):
         """Test execution of a remote command that doesn't request a PTY"""
