@@ -421,6 +421,35 @@ class _TestConnection(ServerTestCase):
         with self.assertRaises(OSError):
             await asyncssh.connect('0.0.0.1', agent_path=None)
 
+    @asynctest
+    async def test_connect_timeout_exceeded(self):
+        """Test connect timeout exceeded"""
+
+        with self.assertRaises(asyncio.TimeoutError):
+            await asyncssh.connect('223.255.255.254', connect_timeout=1)
+
+    @asynctest
+    async def test_connect_timeout_exceeded_string(self):
+        """Test connect timeout exceeded with string value"""
+
+        with self.assertRaises(asyncio.TimeoutError):
+            await asyncssh.connect('223.255.255.254', connect_timeout='0m1s')
+
+    @asynctest
+    async def test_connect_timeout_exceeded_tunnel(self):
+        """Test connect timeout exceeded"""
+
+        with self.assertRaises(asyncio.TimeoutError):
+            await asyncssh.listen(server_host_keys=['skey'],
+                                  tunnel='223.255.255.254', connect_timeout=1)
+
+    @asynctest
+    async def test_invalid_connect_timeout(self):
+        """Test invalid connect timeout"""
+
+        with self.assertRaises(ValueError):
+            await self.connect(connect_timeout=-1)
+
     @asynctest
     async def test_connect_tcp_keepalive_off(self):
         """Test connecting with TCP keepalive disabled"""
